from prompt_toolkit.validation import ValidationError


def validate(value, schema):
    vartype = schema.get("type", "string")
    if vartype != "string":
        return False, None

    max_length = schema.get("max_length", None)
    if max_length and len(value) > max_length:
        raise ValidationError(message=f"Answer cannot be longer than {max_length} characters")
    return False, None



from prompt_toolkit.validation import ValidationError

from ..utils import string_as_bool


def validate(value, schema):
    vartype = schema.get("type", "string")
    if vartype != "boolean":
        return False, None

    try:
        return True, string_as_bool(value)
    except ValueError:
        raise ValidationError(message="Answer must be a boolean (true/false, yes/no)") from None



from prompt_toolkit.validation import ValidationError


def validate(value, schema):
    vartype = schema.get("type", "string")
    if vartype != "string":
        return False, None

    min_length = schema.get("min_length", None)
    if min_length and len(value) < min_length:
        raise ValidationError(message=f"Answer cannot be shorter than {min_length} characters")
    return False, None



import atexit
import os
import shutil
import subprocess
import sys
import tempfile

from . import directory


def git_clone(url):
    git = shutil.which("git")
    if git is None:
        sys.exit("error: git executable was not found")

    # Create a temporary folder to be deleted at exit
    tmpdir = tempfile.mkdtemp(prefix="scaffold")

    def remove_tmpdir():
        shutil.rmtree(tmpdir)

    atexit.register(remove_tmpdir)
    try:
        git = shutil.which("git")
        subprocess.run([git, "clone", url, "repository", "--depth", "1"], cwd=tmpdir, check=True)

        template_dir = os.path.join(tmpdir, "repository")
        return directory.find(template_dir)
    except subprocess.CalledProcessError:
        sys.exit(f'error: failed to clone remote repository "{url}"')


def find(path):
    # Clone the repository
    #
    # TODO: We should not rely on the existance of git,
    # but rather detect, in order, from the url path if
    # it fits common sites (github.com, gitlab.com), if
    # it's prefixed such as git+http[s]://
    #

    return git_clone(path)



"""
Main command-line interface
"""
import sys

from imxdparser import ChildParser, MainParser

from . import clone, validate


def _parser_error(parser, argv, *_args):
    """
    Reparse with an additional parameter to force an error
    """
    argv += [""]
    parser.parse_args(argv)


def _options(argv=None):
    """
    Prepare an intermixed argparser. Command line argument positions
    is a bit more flexible.
    """
    if argv is None:
        argv = sys.argv[1:]

    def error(*_args):
        _parser_error(parser, argv)

    parser = MainParser(description="Lorem ipsum sit dolor amet")
    parser.add_argument("--version", action="version", version="1.4.4")
    parser.attach()
    parser.set_defaults(func=error)

    subparser = ChildParser(parser, "version")
    subparser.attach()
    subparser.set_defaults(func=lambda *_: print("1.4.4"))

    subparser = ChildParser(parser, "validate")
    # fmt: off
    subparser.add_argument("template", metavar="TEMPLATE", help="Path of a 'scaffold.yml' template file")
    # fmt: on
    subparser.attach()
    subparser.set_defaults(func=validate.validate)

    subparser = ChildParser(parser, "clone")
    # fmt: off
    subparser.add_argument("-i", "--input-file", metavar="FILE", help="Location of a yaml input file with answers to the questions")
    subparser.add_argument("-o", "--output-dir", default=".", metavar="PATH", help="Where to output the generated files")
    subparser.add_argument("template", metavar="TEMPLATE", help="Path of a 'scaffold.yml' template file")
    # fmt: on
    subparser.attach()
    subparser.set_defaults(func=clone.clone)

    return vars(parser.parse_args(argv))


def main():
    args = _options()
    if args.get("func"):
        func = args.pop("func")
        func(args)



def string_as_bool(value):
    if isinstance(value, bool):
        return value
    if isinstance(value, int):
        return bool(value)

    lvalue = value.lower().strip()
    if lvalue in ["true", "tru", "tr", "t", "yes", "y"]:
        return True
    if lvalue in ["false", "fals", "fal", "fa", "f", "no", "n"]:
        return False
    raise ValueError(f"Expected boolean value, got '{value}' instead.")



from prompt_toolkit.validation import ValidationError


def validate(value, schema):
    # ignore ctrl-d
    if value is None:
        return False, None
    # value must be "" to set a default
    if value != "":
        return False, None

    default = schema.get("default")
    if default is not None:
        return False, default

    # there's no default, yet answer was "", so
    # show an error message
    nullable = schema.get("nullable", False)
    if nullable:
        raise ValidationError(message="Answer, or empty (ctrl-d), is required")
    else:
        raise ValidationError(message="Answer is required")
